"use strict";

var Promise = require("bluebird");

var logger = require('./logger');

var request = require('request');

var ConfigurationType = require('./ConfigurationType');
var LayoutData = require('./map/data/LayoutData');
var Project = require('./map/data/Project');

var Alias = require('./map/data/Alias');
var LayoutAlias = require('./map/data/LayoutAlias');
var LayoutReaction = require('./map/data/LayoutReaction');
var Reaction = require('./map/data/Reaction');

/**
 * This object contains methods that will communicate with server.
 */
var ServerConnector = {};

ServerConnector._configurationParam = [];
/**
 * List of {@link OverlayCollection} that was added to the server connector. It
 * describes list of all overlays that communicate with server.
 */
ServerConnector._overlays = [];

/**
 * Associative array with methods that should be used for comunication of the
 * {@link OverlayCollection} from _overlays field.
 */
ServerConnector._overlayMethods = [];

/**
 * Form where fields passed to/from server are stored.
 */
ServerConnector.formIdentifier = "_gmapForm";

/**
 * Variable describing when was last transmition of data (location of the map,
 * zoom level, etc.) to the server session.
 */
ServerConnector.lastActualization = 0;

ServerConnector._customMap = null;

/**
 * Get name of the file that should be used as logo.
 */
ServerConnector.getLogoImg = function() {
  return document.getElementById('logoImg').value;
};

/**
 * Get link to the website that should be accessed when clicking on
 * {@link ServerConnector.getLogoImg}.
 */
ServerConnector.getLogoLink = function() {
  return document.getElementById('logoLink').value;
};

/**
 * Get zoom level of the {@link CustomMap} at which it was last browsed.
 */
ServerConnector.getZoomLevel = function() {
  return document.getElementById(ServerConnector.formIdentifier + ':zoomLevel').value;
};

/**
 * Set zoom level for {@link CustomMap} on the server side.
 */
ServerConnector.setZoomLevel = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':zoomLevel').value = "" + value;
};

/**
 * Set x coordinate for {@link CustomMap} on the server side.
 */
ServerConnector.setCenterCoordinateX = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':centerCoordinateX').value = "" + value;
};

/**
 * Get x coordinate of the {@link CustomMap} at which it was last browsed.
 */
ServerConnector.getCenterCoordinateX = function() {
  var result = parseFloat(document.getElementById(ServerConnector.formIdentifier + ':centerCoordinateX').value);
  return result;
};

/**
 * Set y coordinate for {@link CustomMap} on the server side.
 */
ServerConnector.setCenterCoordinateY = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':centerCoordinateY').value = "" + value;
};

/**
 * Get y coordinate of the {@link CustomMap} at which it was last browsed.
 */
ServerConnector.getCenterCoordinateY = function() {
  var result = parseFloat(document.getElementById(ServerConnector.formIdentifier + ':centerCoordinateY').value);
  return result;
};

/**
 * Set layout that is currently browsed for {@link CustomMap} on the server
 * side.
 */
ServerConnector.setSelectedLayout = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':selectedLayout').value = "" + value;
};
/**
 * Sets list of layouts visualized by javascript.
 */
ServerConnector.setVisibleLayouts = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':visibleLayouts').value = value;
};

/**
 * Get layout of the {@link CustomMap} which was last browsed.
 */
ServerConnector.getSelectedLayout = function() {
  var result = document.getElementById(ServerConnector.formIdentifier + ':selectedLayout').value;
  if (result === undefined) {
    result = null;
  }
  return result;
};

/**
 * Sends data about coordinates/zoom level etc. to the server. Method checks if
 * the transmition is not done to often (at most once per second)
 */
ServerConnector.actualizeSessionData = function() {
  var timestamp = new Date().getTime();
  // we send the data with at least 1 second time distance
  if (timestamp > ServerConnector.lastActualization) {
    ServerConnector.lastActualization = timestamp + 1000;
    setTimeout(function() {
      ServerConnector.actualizeParams();
    }, 1000);
  }
};

/**
 * Sends data about coordinates/zoom level etc. to the server. Sending is
 * performed immediatelly.
 * 
 * @see ServerConnector.actualizeSessionData
 */
ServerConnector.actualizeParams = function() {
  _actualizeParams();
};

/**
 * Sets search query that will be handled by server.
 */
ServerConnector.setSearchQuery = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':mapParam').value = "" + value;
};

/**
 * Sets data mining query that will be handled by server.
 */
ServerConnector.setDataMiningQuery = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':missingConnectionParam').value = "" + value;
};

/**
 * Sets drug search query that will be handled by server.
 */
ServerConnector.setDrugQuery = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':drugParam').value = "" + value;
};

/**
 * Sets comment query that will be handled by server.
 */
ServerConnector.setCommentQuery = function(value) {
  logger.error("ServerConnector.setCommentQuery: Not implemented");
  document.getElementById(ServerConnector.formIdentifier + ':commentParam').value = "" + value;
};

/**
 * Sets selected polygon. This might be in futer used oto extract/analyze data
 * in the polygon by the server side.
 */
ServerConnector.setSelectedPolygon = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':selectedPolygon').value = "" + value;
  document.getElementById(ServerConnector.formIdentifier + ':selectedDrugPolygon').value = "" + value;
};

/**
 * Sets identifier of the model where selected polygon (@see
 * ServerConnector.setSelectedPolygon) was selected.
 */
ServerConnector.setExportModelId = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':exportModelId').value = "" + value;
};

/**
 * Sets flag marking if the comments are visible or not.
 */
ServerConnector.setShowComments = function(value) {
  _setShowComments([ {
    name : 'showComments',
    value : value
  } ]);
};

/**
 * Search for similar data mining entries on the map.
 */
ServerConnector.showSimilarDataMining = function(param) {
  document.getElementById(ServerConnector.formIdentifier + ':geneName').value = "" + param;
  _searchMissingConnections();
};

/**
 * Sends request to server to refresh data for overlay with a given name.
 * 
 * @param overlayName
 *          name of the overlay
 */
ServerConnector.sendRefreshRequest = function(overlayName) {
  var functions = ServerConnector._overlayMethods[overlayName];
  if (functions !== undefined) {
    var refreshFunction = functions.refreshFunction;
    if (refreshFunction !== undefined) {
      refreshFunction();
    } else {
      logger.warn("Refresh function for " + overlayName + " doesn't exist");
    }
  } else {
    logger.warn("[Refresh function] Unknown overlay: ", overlayName);
  }
};

/**
 * Adds {@link OverlayCollection} to ServerConnector.
 */
ServerConnector.addOverlayCollection = function(overlay) {
  if (ServerConnector._overlayMethods[overlay.name] !== undefined) {
    if (ServerConnector._overlays[overlay.name] !== undefined) {
      logger.warn("Overlay with name " + overlay.name + " already added");
      return;
    }
    ServerConnector._overlays[overlay.name] = overlay;
    if (ServerConnector._overlayMethods[overlay.name].initFunction !== undefined) {
      ServerConnector._overlayMethods[overlay.name].initFunction();
    } else {
      logger.warn("Cannot initialize overlay: " + overlay);
    }
  } else {
    logger.warn("Unknown overlay: ", overlay);
  }
};

/**
 * Method that should be called by server to update data in overlay.
 * 
 * @param overlayName
 *          name of overlay for which the data is sent
 * @param data
 *          new data for overlay
 * @param fitBounds
 *          should the map call fitBounds after update
 */
ServerConnector.updateOverlayCollection = function(overlayName, data, fitBounds) {
  logger.debug("Updating: " + overlayName);
  var overlay = ServerConnector._overlays[overlayName];
  if (overlay !== undefined) {
    overlay.updateOverlays(data, fitBounds);
  } else {
    logger.warn("Overlay " + overlayName + " cannot be updated. It doesn't exist");
  }
};

/**
 * Name of the overlay for 'search' overlay.
 */
ServerConnector.SEARCH_OVERLAY_NAME = 'search';

/**
 * Name of the overlay for 'data mining' overlay.
 */
ServerConnector.DATA_MINING_OVERLAY_NAME = 'missingConnection';

/**
 * Name of the overlay for 'drug' overlay.
 */
ServerConnector.DRUG_OVERLAY_NAME = 'drug';

/**
 * Name of the overlay for 'chemical' overlay.
 */
ServerConnector.CHEMICAL_OVERLAY_NAME = 'chemical';

/**
 * Name of the overlay for 'comment' overlay.
 */
ServerConnector.COMMENT_OVERLAY_NAME = 'comment';

/**
 * Name of the overlay for 'mirna' overlay.
 */
ServerConnector.MI_RNA_OVERLAY_NAME = 'mirna';

/**
 * Register 'search' overlay on the server.
 */
ServerConnector.registerSearchOverlay = function() {
  _registerSearchOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.SEARCH_OVERLAY_NAME,
  } ]);
};

/**
 * Sends request to the server to refresh data in 'search' overlay.
 */
ServerConnector.refreshSearchOverlay = function() {
  _refreshSearchOverlayCollection();
};

/**
 * Clear data related to 'search' overlay..
 */
ServerConnector.clearSearchOverlay = function() {
  _clearSearchOverlayCollection();
};

/**
 * Register 'data mining' overlay on the server.
 */
ServerConnector.registerDataMiningOverlay = function() {
  _registerMissingConnectionOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.DATA_MINING_OVERLAY_NAME,
  } ]);
};

/**
 * Sends request to the server to refresh data in 'data mining' overlay.
 */
ServerConnector.refreshDataMiningOverlay = function() {
  _refreshMissingConnectionOverlayCollection();
};

/**
 * Clear data related to 'data mining' overlay.
 */
ServerConnector.clearDataMiningOverlay = function() {
  _clearMissingConnectionOverlayCollection();
};

/**
 * Register 'drug' overlay on the server.
 */
ServerConnector.registerDrugOverlay = function() {
  _registerDrugOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.DRUG_OVERLAY_NAME,
  } ]);
};

ServerConnector.requestDrugDetailDataFunction = function(identfiedElement, id) {
  logger.debug("Requesting drug details: ", identfiedElement, id);
  _requestDrugDetailDataFunction([ {
    name : "searchResultIdentifier",
    value : id,
  }, {
    name : "objectIdentifier",
    value : identfiedElement.getId(),
  }, {
    name : "modelIdentifier",
    value : identfiedElement.getModelId(),
  }, {
    name : "type",
    value : identfiedElement.getType(),
  } ]);
};
ServerConnector.requestCommentDetailDataFunction = function(identfiedElement, id) {
  _requestCommentDetailDataFunction([ {
    name : "searchResultIdentifier",
    value : id,
  }, {
    name : "objectIdentifier",
    value : identfiedElement.getId(),
  }, {
    name : "modelIdentifier",
    value : identfiedElement.getModelId(),
  }, {
    name : "type",
    value : identfiedElement.getType(),
  } ]);
};

/**
 * Sends request to the server to refresh data in 'drug' overlay.
 */
ServerConnector.refreshDrugOverlay = function() {
  _refreshDrugOverlayCollection();
};

/**
 * Clear data related to 'drug' overlay.
 */
ServerConnector.clearDrugOverlay = function() {
  _clearDrugOverlayCollection();
};

ServerConnector.registerChemicalOverlay = function() {
  _registerChemicalOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.CHEMICAL_OVERLAY_NAME,
  } ]);
};

ServerConnector.requestChemicalDetailDataFunction = function(identfiedElement, id) {
  _requestChemicalDetailDataFunction([ {
    name : "searchResultIdentifier",
    value : id,
  }, {
    name : "objectIdentifier",
    value : identfiedElement.getId(),
  }, {
    name : "modelIdentifier",
    value : identfiedElement.getModelId(),
  }, {
    name : "type",
    value : identfiedElement.getType(),
  } ]);
};

/**
 * Sends request to the server to refresh data in 'drug' overlay.
 */
ServerConnector.refreshChemicalOverlay = function() {
  _refreshChemicalOverlayCollection();
};

/**
 * Clear data related to 'chemical' overlay.
 */
ServerConnector.clearChemicalOverlay = function() {
  _clearChemicalOverlayCollection();
};

/**
 * Register 'comment' overlay on the server.
 */
ServerConnector.registerCommentOverlay = function() {
  _registerCommentOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.COMMENT_OVERLAY_NAME,
  } ]);
};

/**
 * Sends request to the server to refresh data in 'comment' overlay.
 */
ServerConnector.refreshCommentOverlay = function() {
  _refreshCommentOverlayCollection();
};

/**
 * Clear data related to 'comment' overlay.
 */
ServerConnector.clearCommentOverlay = function() {
  _clearCommentOverlayCollection();
};

/**
 * Clear data related to 'data mining' overlay.
 */
ServerConnector.clearDataMiningOverlay = function() {
  _clearMissingConnectionOverlayCollection();
};

/**
 * Register 'drug' overlay on the server.
 */
ServerConnector.registerMiRnaOverlay = function() {
  _registerMiRnaOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.MI_RNA_OVERLAY_NAME,
  } ]);
};

ServerConnector.requestMiRnaDetailDataFunction = function(identfiedElement, id) {
  _requestMiRnaDetailDataFunction([ {
    name : "searchResultIdentifier",
    value : id,
  }, {
    name : "objectIdentifier",
    value : identfiedElement.getId(),
  }, {
    name : "modelIdentifier",
    value : identfiedElement.getModelId(),
  }, {
    name : "type",
    value : identfiedElement.getType(),
  } ]);
};

/**
 * Sends request to the server to refresh data in 'drug' overlay.
 */
ServerConnector.refreshMiRnaOverlay = function() {
  _refreshMiRnaOverlayCollection();
};

/**
 * Clear data related to 'mirna' overlay.
 */
ServerConnector.clearMiRnaOverlay = function() {
  _clearMiRnaOverlayCollection();
};

/**
 * Define pack of methods for 'search' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.SEARCH_OVERLAY_NAME] = {
    initFunction : ServerConnector.registerSearchOverlay,
    refreshFunction : ServerConnector.refreshSearchOverlay,
    clearFunction : ServerConnector.clearSearchOverlay,
};

/**
 * Define pack of methods for 'data minig' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.DATA_MINING_OVERLAY_NAME] = {
    initFunction : ServerConnector.registerDataMiningOverlay,
    refreshFunction : ServerConnector.refreshDataMiningOverlay,
    clearFunction : ServerConnector.clearDataMiningOverlay,
};

/**
 * Define pack of methods for 'drug' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.DRUG_OVERLAY_NAME] = {
    initFunction : ServerConnector.registerDrugOverlay,
    refreshFunction : ServerConnector.refreshDrugOverlay,
    clearFunction : ServerConnector.clearDrugOverlay,
    requestDetailDataFunction : ServerConnector.requestDrugDetailDataFunction,
};

/**
 * Define pack of methods for 'chemical' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.CHEMICAL_OVERLAY_NAME] = {
    initFunction : ServerConnector.registerChemicalOverlay,
    refreshFunction : ServerConnector.refreshChemicalOverlay,
    clearFunction : ServerConnector.clearChemicalOverlay,
    requestDetailDataFunction : ServerConnector.requestChemicalDetailDataFunction,
};

/**
 * Define pack of methods for 'comment' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.COMMENT_OVERLAY_NAME] = {
    initFunction : ServerConnector.registerCommentOverlay,
    refreshFunction : ServerConnector.refreshCommentOverlay,
    clearFunction : ServerConnector.clearCommentOverlay,
    requestDetailDataFunction : ServerConnector.requestCommentDetailDataFunction,
};

ServerConnector._overlayMethods[ServerConnector.MI_RNA_OVERLAY_NAME] = {
    initFunction : ServerConnector.registerMiRnaOverlay,
    refreshFunction : ServerConnector.refreshMiRnaOverlay,
    clearFunction : ServerConnector.clearMiRnaOverlay,
    requestDetailDataFunction : ServerConnector.requestMiRnaDetailDataFunction,
};

ServerConnector.sendClearRequest = function(overlayName) {
  var functions = ServerConnector._overlayMethods[overlayName];
  if (functions !== undefined) {
    var clearFunction = functions.clearFunction;
    if (clearFunction !== undefined) {
      clearFunction();
    } else {
      logger.warn("Clear function for " + overlayName + " doesn't exist");
    }
  } else {
    logger.warn("[Clear function] Unknown overlay: ", overlayName);
  }
};

ServerConnector.setOverlayResultIds = function(overlayName, ids) {
  var overlay = this._overlays[overlayName];
  if (overlay !== undefined) {
    overlay.setResultsIds(ids);
  } else {
    logger.warn("Overlay '" + overlayName + "' doesn't exist");
  }
};

ServerConnector.sendOverlayDetailDataRequest = function(overlayName, identifiedElement, general) {
  var overlay = ServerConnector._overlays[overlayName];
  if (overlay === undefined) {
    logger.warn("Unknown overlay: " + overlayName);
    return;
  }
  var functions = ServerConnector._overlayMethods[overlayName];
  if (functions !== undefined) {
    var requestDetailDataFunction = functions.requestDetailDataFunction;
    if (requestDetailDataFunction !== undefined) {
      var missingData = overlay.getMissingDetailDataIds(identifiedElement, general);
      if (missingData.length === 0) {
        logger.warn("request of missing data called for empty data set... overlay name = ", overlayName, "general: ",
            general);
      }
      for (var i = 0; i < missingData.length; i++) {
        requestDetailDataFunction(identifiedElement, missingData[i]);
      }
    } else {
      logger.warn("Request detail data function for " + overlayName + " doesn't exist");
    }
  } else {
    logger.warn("[Clear function] Unknown overlay: ", overlayName);
  }

};

ServerConnector.updateOverviewElementDetailData = function(overlayName, searchResultIdentifier, identifiedElement, data) {
  var overlay = this._overlays[overlayName];
  if (overlay !== undefined) {
    overlay.updateOverviewElementDetailData(identifiedElement, searchResultIdentifier, data);
  } else {
    logger.warn("Overlay '" + overlayName + "' doesn't exist");
  }
};

ServerConnector.setCustomMap = function(customMap) {
  this._customMap = customMap;
};

ServerConnector.getCustomMap = function() {
  return this._customMap;
};

ServerConnector.addAliasesForLayout = function(layoutId, jsonAliases) {
  this.setWaitingForAliasLayoutData(parseInt(layoutId), false);
  this.getCustomMap().addAliasesForLayout(layoutId, jsonAliases);
};

ServerConnector.updateAliasesForLayout = function(layoutId, jsonAliases) {
  var arr = JSON.parse(jsonAliases);
  for (var i = 0; i < arr.length; i++) {
    this.setWaitingForFullAliasLayoutData(parseInt(layoutId), parseInt(arr[i].idObject), false);
  }
  this.getCustomMap().updateAliasesForLayout(layoutId, jsonAliases);
};

ServerConnector.addAliases = function(aliases) {
  for (var i = 0; i < aliases.length; i++) {
    this.setWaitingForAliasData(parseInt(aliases[i].idObject), false);
  }
  this.getCustomMap().addAliases(aliases);
};

ServerConnector.addReactionsForLayout = function(layoutId, jsonReactions) {
  this.setWaitingForReactionLayoutData(parseInt(layoutId), false);
  this.getCustomMap().addReactionsForLayout(layoutId, jsonReactions);
};

ServerConnector.addReactions = function(jsonReactions) {
  var arr = JSON.parse(jsonReactions);
  for (var i = 0; i < arr.length; i++) {
    this.setWaitingForReactionData(parseInt(arr[i].idObject), false);
  }
  this.getCustomMap().addReactions(jsonReactions);
};

ServerConnector.searchByCoord = function(modelId, latLngCoordinates) {
  _searchByCoord([ {
    name : 'submodelId',
    value : modelId
  }, {
    name : 'latCoord',
    value : latLngCoordinates.lat()
  }, {
    name : 'lngCoord',
    value : latLngCoordinates.lng()
  } ]);
};

ServerConnector.requestUpdateCommentList = function(modelId, latLngCoordinates) {
  _updateCommentList([ {
    name : 'submodelId',
    value : modelId
  }, {
    name : 'latCoord',
    value : latLngCoordinates.lat()
  }, {
    name : 'lngCoord',
    value : latLngCoordinates.lng()
  } ]);
};

ServerConnector.sendReferenceGenomeDetailRequest = function(type, version) {
  logger.debug("Send request", type, version);
  _sendReferenceGenomeDetailRequest([ {
    name : 'type',
    value : type
  }, {
    name : 'version',
    value : version
  } ]);
};
ServerConnector.updateReferenceGenomeData = function(organismId, type, version, jsonObj) {
  this.getCustomMap().updateReferenceGenome(type, version, jsonObj);
};

ServerConnector.getMinOverlayColorInt = function() {
  var color = document.getElementById("overlayConfigForm:userMinColor").value;
  if (color === "") {
    color = document.getElementById("overlayConfigForm:systemMinColor").value;
  }
  color = parseInt(color, 16);
  /* jslint bitwise: true */
  color = (color & 0xFFFFFF);
  return color;
};

ServerConnector.getMaxOverlayColorInt = function() {
  var color = document.getElementById("overlayConfigForm:userMaxColor").value;
  if (color === "") {
    color = document.getElementById("overlayConfigForm:systemMaxColor").value;
  }
  color = parseInt(color, 16);
  /* jslint bitwise: true */
  color = (color & 0xFFFFFF);
  return color;
};

ServerConnector.createSubmodelDialog = function(submodelId) {
  _createSubmodelDialog([ {
    name : 'submodelId',
    value : submodelId
  } ]);
};

ServerConnector.setModelZoomLevel = function(id, zoom) {
  _setModelZoomLevel([ {
    name : 'mapId',
    value : id
  }, {
    name : 'zoomLevel',
    value : zoom
  } ]);
};

ServerConnector.readFile = function(url) {
  return new Promise(function(resolve, reject) {
    request.get(url, function(error, response, body) {
      if (error) {
        reject(error);

      } else if (response.statusCode !== 200) {
        reject(response);
      } else {
        resolve(body);
      }
    });
  });
};

ServerConnector.getToken = function() {
  var self = this;
  return new Promise(function(resolve) {
    if (self._token === undefined) {
      self.setToken(document.getElementById("authenticationForm:authenticationToken").value);
    }
    resolve(self._token);
  });
};

ServerConnector.setToken = function(token) {
  this._token = token;
};

ServerConnector.getApiBaseUrl = function() {
  return this.getServerBaseUrl()+"/api/";
};

ServerConnector.getServerBaseUrl = function() {
  if (this._serverBaseUrl === undefined) {
    var location = window.location;
    this._serverBaseUrl = location.protocol + '//' + location.hostname + (location.port ? ':' + location.port : '')
    + document.getElementById("authenticationForm:baseUrl").value;
  }
  return this._serverBaseUrl;
};

ServerConnector.createGetParams = function(params) {
  var sorted = [], key;
  
  for (key in params) {
      if (params.hasOwnProperty(key)) {
        sorted.push(key);
      }
  }
  sorted.sort();
  
  var result = "";
  for (var i = 0; i < sorted.length; i++) {
    result+= sorted[i] + "="+params[sorted[i]]+"&";
  }
  return result;
};

ServerConnector.getApiUrl = function(paramObj) {
  var type = paramObj.type;
  var method = paramObj.method;
  var params = this.createGetParams(paramObj.params);

  var result = this.getApiBaseUrl() + "/"+type+"/"+method+"?"+params;
  return result;
};


ServerConnector.getProjectUrl = function(projectId, token) {
  return this.getApiUrl({type:"project",
    method: "getMetaData",
    params: {
      projectId: projectId,
      token: token,
    },
  });
};

ServerConnector.getOverlaysUrl = function(projectId, token) {
  return this.getApiUrl({type:"overlay",
    method: "getOverlayList",
    params: {
      projectId: projectId,
      token: token,
    },
  });
};

ServerConnector.getOverlayByIdUrl = function(overlayId, projectId, token) {
  return this.getApiUrl({type:"overlay",
    method: "getOverlayById",
    params: {
      projectId: projectId,
      token: token,
      overlayId: overlayId,
    },
  });
};

ServerConnector.getOverlayElementsUrl = function(overlayId, projectId, token) {
  return this.getApiUrl({type:"overlay",
    method: "getOverlayElements",
    params: {
      projectId: projectId,
      token: token,
      overlayId: overlayId,
    },
  });
};

ServerConnector.idsToString = function (ids) {
  var result = "";
  for (var i = 0; i < ids.length; i++) {
    if (result !== "") {
      result = result + "," + ids[i];
    } else {
      result = ids[i];
    }
  }
  return result;
};

ServerConnector.columnsToString = function (columns) {
  if (columns === undefined) {
    return "";
  }
  return columns;
};

ServerConnector.getReactionsUrl = function(reactionIds, projectId, token, columns) {
  var id = this.idsToString(reactionIds);
  columns = this.columnsToString(columns);

  return this.getApiUrl({type:"project",
    method: "getReactions",
    params: {
      projectId: projectId,
      token: token,
      columns: columns,
      id: id,
    },
  });
};

ServerConnector.getAliasesUrl = function(params) {
  var id = this.idsToString(params.ids);
  var columns = this.columnsToString(params.columns);
  var projectId = params.projectId;
  var token = params.token;

  return this.getApiUrl({type:"project",
    method:"getElements",
    params: {
      projectId: projectId, 
      columns: columns, 
      id: id, 
      token: token},
  });
};

ServerConnector.getConfigurationUrl = function(token) {
  var result = this.getApiUrl({type:"configuration",
    method: "getAllValues",
    params: {
      token: token,
    },
  });
  return result;
};

ServerConnector.getConfigurationParam = function(paramId) {
  var self = this;
  return new Promise(function(resolve, reject) {
    if (paramId === undefined) {
      reject("Invalid param identifier");
    } else      if (self._configurationParam[paramId] !== undefined) {
      resolve(self._configurationParam[paramId]);
    } else {
      return self.getToken().then(function(token) {
        return self.readFile(self.getConfigurationUrl(token));
      }).then(function(content) {
        var configs = JSON.parse(content);
        for (var i = 0; i < configs.length; i++) {
          var conf = configs[i];
          var type = conf.type;
          var value = conf.value;
          self._configurationParam[type] = value;
        }
        if (self._configurationParam[paramId] === undefined) {
          reject("Cannot find param config: " + paramId);
        }
        resolve(self._configurationParam[paramId]);
      });
    }
  });
};

ServerConnector.getProject = function(projectId) {
  var self = this;
  if (projectId === undefined || projectId === null || projectId === "") {
    return new Promise(function(resolve, reject) {
      self.getConfigurationParam(ConfigurationType.DEFAULT_MAP).then(function(defaultMap) {
        self.getProject(defaultMap).then(function(project) {
          resolve(project);
        }, reject);
      }, reject);
    });
  }
  return new Promise(function(resolve, reject) {
    self.getToken().then(function(token) {
      self.readFile(self.getProjectUrl(projectId, token)).then(function(content) {
        var project = new Project(content);
        self.getOverlays(projectId).then(function(overlays) {
          project.getModel().addLayouts(overlays);
          resolve(project);
        }, reject);
      }, reject);
    }, reject);
  });
};

ServerConnector.getOverlays = function(projectId) {
  var self = this;
  if (projectId === undefined || projectId === null || projectId === "") {
    return new Promise(function(resolve, reject) {
      self.getConfigurationParam(ConfigurationType.DEFAULT_MAP).then(function(defaultMap) {
        self.getOverlays(defaultMap).then(function(overlays) {
          resolve(overlays);
        }, reject);
      }, reject);
    });
  }
  return new Promise(function(resolve, reject) {
    self.getToken().then(function(token) {
      self.readFile(self.getOverlaysUrl(projectId, token)).then(function(content) {
        var arr = JSON.parse(content);
        var result = [];
        for (var i = 0; i < arr.length; i++) {
          var layout = new LayoutData(arr[i]);
          result.push(layout);
        }
        resolve(result);
      }, reject);
    }, reject);
  });
};

ServerConnector.getOverlayElements = function(layoutId, projectId) {
  var self = this;
  if (layoutId === undefined) {
    throw new Error("Layout id must be defined");
  }
  return new Promise(function(resolve, reject) {
    self.getToken().then(function(token) {
      self.getProjectId(projectId).then(function(projectId) {
        self.readFile(self.getOverlayElementsUrl(layoutId, projectId, token)).then(function(content) {
          var arr = JSON.parse(content);
          var result = [];
          for (var i = 0; i < arr.length; i++) {
            var element = arr[i];
            if (element.type === "REACTION") {
              result.push(new LayoutReaction(element.overlayContent));
            } else if (element.type === "ALIAS") {
              result.push(new LayoutAlias(element.overlayContent));
            } else {
              reject("Unknown element type: " + element.type);
            }
          }
          resolve(result);
        }, reject);
      }, reject);
    }, reject);
  });
};

ServerConnector.getProjectId = function(projectId) {
  var self = this;
  if (projectId === undefined || projectId === null || projectId === "") {
    return new Promise(function(resolve, reject) {
      self.getConfigurationParam(ConfigurationType.DEFAULT_MAP).then(function(defaultMap) {
        resolve(defaultMap);
      }, reject);
    });
  } else {
    return new Promise(function(resolve) {
      resolve(projectId);
    });
  }
};

ServerConnector.getOverlayById = function(layoutId, projectId) {
  var self = this;
  return new Promise(function(resolve, reject) {
    self.getProjectId(projectId).then(function(projectId) {
      self.getToken().then(function(token) {
        self.readFile(self.getOverlayByIdUrl(layoutId, projectId, token)).then(function(content) {
          var result = new LayoutData(JSON.parse(content));
          resolve(result);
        }, reject);
      }, reject);
    }, reject);
  });
};

ServerConnector.getReactions = function(reactionIds, projectId, columns) {
  var self = this;
  return new Promise(function(resolve, reject) {
    self.getProjectId(projectId).then(function(result) {
      projectId = result;
      return self.getToken();
    }).then(function(token) {
      return self.readFile(self.getReactionsUrl(reactionIds, projectId, token, columns));
    }).then(function(content) {
      var array=JSON.parse(content);
      var result = [];
      for (var i = 0; i < array.length; i++) {
        result.push(new Reaction(array[i]));
      }
      resolve(result);
    }).catch(function(exception){
      reject(exception);
    });
  });
};

ServerConnector.getAliases = function(aliasIds, projectId, columns) {
  var self = this;
  return new Promise(function(resolve, reject) {
    self.getProjectId(projectId).then(function(result) {
      projectId = result;
      return self.getToken();
    }).then(function(token) {
      return self.readFile(self.getAliasesUrl({ids:aliasIds, projectId:projectId, token:token, columns:columns}));
    }).then(function(content) {
      var array=JSON.parse(content);
      var result = [];
      for (var i = 0; i < array.length; i++) {
        result.push(new Alias(array[i]));
      }
      resolve(result);
    }).catch(function(exception){
      reject(exception);
    });
  });
};

ServerConnector.getLightAliases = function(aliasIds, projectId) {
  return this.getAliases(aliasIds,projectId,"id,bounds,modelId");
};

module.exports = ServerConnector;
