"use strict";

var logger = require('../../logger');

var IdentifiedElement = require('../data/IdentifiedElement');
var AbstractMarker = require('../marker/AbstractMarker');
var ObjectWithListeners = require('../../ObjectWithListeners');

/**
 * Class representing any info window in our map.
 * 
 * @param abstractMarker
 *          {@link AbstractMarker} to which this window is pointing, can be null
 */
function AbstractInfoWindow(abstractMarker, map) {
  // call super constructor
  ObjectWithListeners.call(this);

  if (abstractMarker instanceof AbstractMarker) {
    this.setCustomMap(abstractMarker.getCustomMap());
    this._marker = abstractMarker;
  } else {
    this.setCustomMap(map);
  }

  this.setContent(this.createWaitingContentDiv());

  this._overlayFullView = [];

  this.registerPropertyType("overlayFullView");

  // listener called when user want to see all data about specific data overlay
  var onOverlayFullViewChanged = function(e) {
    var self = e.object;
    // first change the content of the element
    self.update();
    if (e.newVal) {
      var element = new IdentifiedElement({
        objectId : self.getElementId(),
        modelId : self.getCustomMap().getId(),
        type : self.getElementType()
      });

      self.getCustomMap().getTopMap().retrieveOverlayDetailDataForElement(element, self.getOverlayFullViewArray());
    }

  };

  // default settings of visualizing full information about elements
  this.setOverlayFullView("drug", false);
  this.setOverlayFullView("chemical", false);
  this.setOverlayFullView("mirna", false);
  this.setOverlayFullView("search", false);
  this.setOverlayFullView("missingConnection", false);
  // only all comments should be visible from the beginning
  this.setOverlayFullView("comment", true);

  this.addPropertyChangeListener("overlayFullView", onOverlayFullViewChanged);

  this.registerListenerType("onShow");
  this.registerListenerType("onUpdate");

}

AbstractInfoWindow.prototype = Object.create(ObjectWithListeners.prototype);
AbstractInfoWindow.prototype.constructor = AbstractInfoWindow;

/**
 * Returns <code>true</code> if overlay should visualize all possible values.
 * 
 * @param overlayName
 *          name of the overlay
 * @returns <code>true</code> if overlay should visualize all possible values
 */
AbstractInfoWindow.prototype.isOverlayFullView = function(overlayName) {
  if (this._overlayFullView[overlayName] === undefined) {
    this._overlayFullView[overlayName] = false;
  }
  return this._overlayFullView[overlayName];
};

/**
 * Returns associative array with information if specific overlay should present
 * all possible results or only specified by the data searched by user.
 * 
 * @returns {Array} with information if specific overlay should present all
 *          possible results or only specified by the data searched by user
 */
AbstractInfoWindow.prototype.getOverlayFullViewArray = function() {
  return this._overlayFullView;
};

AbstractInfoWindow.prototype.setOverlayFullView = function(overlayName, value) {
  var oldVal = this._overlayFullView[overlayName];
  this._overlayFullView[overlayName] = value;
  this.firePropertyChangeListener("overlayFullView", overlayName + "," + oldVal, value);
};

/**
 * This method checks if {@link AbstractInfoWindow} is opened.
 * 
 * @returns {Boolean} <code>true</code> if window is opened,
 *          <code>false</code> otherwise
 */
AbstractInfoWindow.prototype.isOpened = function() {
  if (this.googleInfowindow === undefined) {
    return false;
  }
  var infoWindowMap = this.googleInfowindow.getMap();
  if (infoWindowMap !== null && typeof infoWindowMap !== "undefined") {
    return true;
  } else {
    return false;
  }
};

/**
 * Opens Info Window.
 */
AbstractInfoWindow.prototype.open = function() {
  if (this.googleInfowindow === null) {
    logger.warn("Cannot open window.");
    return;
  }
  this.update();
  this.googleInfowindow.open(this.getCustomMap().map, this.getGoogleMarker());

  this.callListeners("onShow");
};

/**
 * Sets new content of the info window.
 * 
 * @param content
 *          new content of the window
 */
AbstractInfoWindow.prototype.setContent = function(content) {
  this._content = content;
  if (this.googleInfowindow !== undefined) {
    this.googleInfowindow.setContent(content);
  }
};

/**
 * Returns content visualized in the info window.
 * 
 * @returns {String} content visualized in the info window
 */
AbstractInfoWindow.prototype.getContent = function() {
  return this._content;
};

/**
 * Creates div for an overlay data.
 * 
 * @param overlay
 *          corresponding {@link OverlayCollection}
 * @param data
 *          data taken from overlay
 * @returns string representing div for given overlay data
 */
AbstractInfoWindow.prototype.createOverlayInfoDiv = function(overlay, data) {
  if (this.alias !== undefined) {
    if (this.alias.getType() !== undefined) {
      if (overlay.name === "drug") {
        if (this.alias.getType().toUpperCase() === "RNA" || //
        this.alias.getType().toUpperCase() === "PROTEIN" || //
        this.alias.getType().toUpperCase() === "GENE") {
          return this._createDrugInfoDiv(overlay, data);
        } else {
          return null;
        }
      } else if (overlay.name === "chemical") {
        if (this.alias.getType().toUpperCase() === "RNA" || //
        this.alias.getType().toUpperCase() === "PROTEIN" || //
        this.alias.getType().toUpperCase() === "GENE") {
          return this._createChemicalInfoDiv(overlay, data);
        } else {
          return null;
        }
      } else if (overlay.name === "mirna") {
        if (this.alias.getType().toUpperCase() === "RNA" || //
        this.alias.getType().toUpperCase() === "PROTEIN" || //
        this.alias.getType().toUpperCase() === "GENE") {
          return this._createMiRnaInfoDiv(overlay, data);
        } else {
          return null;
        }

      } else if (overlay.name === "comment") {
        return this._createCommentInfoDiv(overlay, data);
      } else {
        logger.warn("Unknown overlay data for AliasInfoWindow: " + overlay.name);
        return this._createDefaultInfoDiv(overlay, data);
      }
    } else {
      logger.warn("Cannot customize info window. Alias type is unknown: ", this.alias.getType());
      return null;
    }
  } else {
    if (overlay.name === "comment") {
      return this._createCommentInfoDiv(overlay, data);
    } else {
      logger.debug("Cannot customize info window. Alias not defined.");
      return null;
    }
  }
};

/**
 * Creates and returns div for drug overlay information.
 * 
 * @param data
 *          data taken from drug iverlay
 * @returns {String} with a div for drug overlay information
 */
AbstractInfoWindow.prototype._createDrugInfoDiv = function(overlay, data) {
  return this._createTargetInfoDiv(overlay, data, "Interacting drugs");
};

/**
 * Creates and returns div for comment overlay information.
 * 
 * @param data
 *          data taken from comment iverlay
 * @returns {String} with a div for comment overlay information
 */
AbstractInfoWindow.prototype._createCommentInfoDiv = function(overlay, data) {
  if (data.length === 0 || data[0] === undefined) {
    return null;
  }
  var result = document.createElement("div");

  var titleElement = document.createElement("h3");
  titleElement.innerHTML = "Comments";
  result.appendChild(titleElement);
  for (var i = 0; i < data.length; i++) {
    if (typeof data[i].commentId !== "undefined" && typeof data[i].commentContent !== "undefined") {
      result.appendChild(document.createElement("hr"));
      var commentId = document.createElement("div");
      commentId.innerHTML = '#' + data[i].commentId;
      result.appendChild(commentId);
      result.appendChild(document.createElement("br"));
      var commentContent = document.createElement("div");
      commentContent.innerHTML = '#' + data[i].commentContent;
      result.appendChild(commentContent);
    } else {
      throw new Error("Invalid comment data: ", data[i]);
    }
  }

  return result;
};

/**
 * Creates and returns div for unknown overlay.
 * 
 * @param overlay
 *          some {@link OverlayCollection}
 * @param data
 *          data taken from overlay
 * @returns Dom element with a div for overlay information
 */

AbstractInfoWindow.prototype._createDefaultInfoDiv = function(overlay, data) {
  var divElement = document.createElement("div");
  var count = 0;

  var titleElement = document.createElement("h3");
  var title = document.createTextNode(overlay.getName());
  titleElement.appendChild(title);
  divElement.appendChild(titleElement);
  for ( var searchId in data) {
    if (data.hasOwnProperty(searchId) && data[searchId] !== undefined && data[searchId] !== null) {
      count++;
      var resultTitleElement = document.createElement("h4");
      var resultTitle = document.createTextNode(searchId);
      resultTitleElement.appendChild(resultTitle);
      divElement.appendChild(resultTitleElement);

      var keys = Object.keys(data[searchId]);
      for (var i = 0; i < keys.length; i++) {
        var resultValElement = document.createElement("p");
        var resultVal = document.createTextNode(keys[i] + ": " + data[searchId][keys[i]]);
        resultValElement.appendChild(resultVal);
        divElement.appendChild(resultValElement);
      }
    }
  }

  if (count === 0) {
    divElement = null;
  }
  return divElement;
};

/**
 * Returns google.maps.Marker object where this info window is attached.
 * 
 * @returns google.maps.Marker object where this info window is attached
 */
AbstractInfoWindow.prototype.getGoogleMarker = function() {
  if (this._marker === undefined) {
    return null;
  } else {
    return this._marker.getGoogleMarker();
  }
};

/**
 * Returns {@link AbstractCustomMap} where this window is presented.
 * 
 * @returns {@link AbstractCustomMap} where this window is presented
 */
AbstractInfoWindow.prototype.getCustomMap = function() {
  return this.customMap;
};

AbstractInfoWindow.prototype.setCustomMap = function(map) {
  if (map===undefined) {
    throw new Error("Map must be defined");
  }
  this.customMap = map;
};

/**
 * Returns html DOM object with content that should presented when waiting for
 * some data from server.
 * 
 * @returns {String} html with content that should presented when waiting for
 *          some data from server
 */
AbstractInfoWindow.prototype.createWaitingContentDiv = function() {
  var result = document.createElement("div");
  var img = document.createElement("img");
  img.src = GuiConnector.getImgPrefix() + GuiConnector.getLoadingImg();
  var message = document.createElement("h4");
  message.innerHTML = "loading...";
  result.appendChild(img);
  result.appendChild(message);
  return result;
};

/**
 * This is a generic method that updates content of the window.
 */
AbstractInfoWindow.prototype._updateContent = function() {
  var content = this.createContentDiv();
  if (content === undefined || content === null) {
    content = this.createWaitingContentDiv();
  }
  var overlaysDiv = this.createOverlaysDiv();
  if (overlaysDiv !== undefined && overlaysDiv !== null) {
    content.appendChild(overlaysDiv);
  }
  this.setContent(content);

  this.callListeners("onUpdate");
};

/**
 * Creates and returns div with overlays content.
 * 
 * @returns {String} with html representing data taken from
 *          {@link OverlayCollection} for this window
 */
AbstractInfoWindow.prototype.createOverlaysDiv = function() {
  var overlayData = this.getOverlaysData(this.getOverlayFullViewArray());
  if (overlayData === undefined || overlayData === null) {
    return null;
  }
  var result = document.createElement("div");
  for (var i = 0; i < overlayData.length; i++) {
    var overlay = overlayData[i].overlay;
    var data = overlayData[i].data;
    var overlayInfo = this.createOverlayInfoDiv(overlay, data);
    if (overlayInfo !== null) {
      result.appendChild(overlayInfo);
    }
  }
  return result;
};

/**
 * Abstract method (to be implemented by subclasses) for updating content.
 */
AbstractInfoWindow.prototype.update = function() {
  this._updateContent();
};

AbstractInfoWindow.prototype._createTargetInfoDiv = function(overlay, data, name) {
  var abstractInfoWindowSelf = this;
  var result = document.createElement("div");

  var count = 0;

  var titleElement = document.createElement("h3");
  titleElement.innerHTML = name;
  result.appendChild(titleElement);
  if (overlay.allowGeneralSearch()) {
    var checkboxDiv = document.createElement("div");
    checkboxDiv.style.float = "right";

    var checkbox = document.createElement("input");
    checkbox.id = "checkbox-" + name + "-" + this.getElementType() + "-" + this.getElementId();
    checkbox.type = "checkbox";
    checkbox.checked = abstractInfoWindowSelf.isOverlayFullView(overlay.getName());
    var checkboxClickedFunction = function() {
      abstractInfoWindowSelf.setOverlayFullView(overlay.getName(), this.checked);
      return true;
    };
    checkbox.onclick = checkboxClickedFunction;

    checkboxDiv.appendChild(checkbox);

    var description = document.createElement("div");
    description.style.float = "right";
    description.innerHTML = "Show all";
    checkboxDiv.appendChild(description);
    result.appendChild(checkboxDiv);
  }
  var table = document.createElement("table");
  table.className = "mapInfoBoxResultsTable";
  var row = document.createElement("tr");
  var headerCol = document.createElement("th");
  headerCol.innerHTML = "Name";
  row.appendChild(headerCol);
  headerCol = document.createElement("th");
  headerCol.innerHTML = "Source";
  row.appendChild(headerCol);
  headerCol = document.createElement("th");
  headerCol.innerHTML = "References";
  row.appendChild(headerCol);

  count = 0;
  for ( var dataId in data) {
    if (data.hasOwnProperty(dataId) && data[dataId] !== undefined && data[dataId] !== null) {
      // if we have empty array, it means that we got data from server and it's
      // empty
      if (Object.prototype.toString.call(data[dataId]) === '[object Array]') {
        continue;
      } else {
        count++;
      }
    }
  }

  var cell;
  if (count > 0 || abstractInfoWindowSelf.isOverlayFullView(overlay.getName())) {
    table.appendChild(row);

    for ( var searchId in data) {
      if (data.hasOwnProperty(searchId) && data[searchId] !== undefined && data[searchId] !== null) {
        // if we have empty array, it means that we got data from server and
        // it's
        // empty
        if (Object.prototype.toString.call(data[searchId]) === '[object Array]') {
          continue;
        } else {
          row = document.createElement("tr");
          cell = document.createElement("td");
          cell.innerHTML = searchId;
          row.appendChild(cell);
          cell = document.createElement("td");
          if (data[searchId].databaseSource !== undefined) {
            cell.innerHTML = GuiConnector.referenceToHtml(data[searchId].databaseSource);
          } else {
            cell.innerHTML = "N/A";
          }
          row.appendChild(cell);
          cell = document.createElement("td");
          var cellContent = "<ul>";
          for (var i = 0; i < data[searchId].references.length; i++) {
            var reference = data[searchId].references[i];
            cellContent += "<li>" + GuiConnector.referenceToHtml(reference) + "</li>";
          }
          cellContent += "</ul>";
          cell.innerHTML = cellContent;
          row.appendChild(cell);
          table.appendChild(row);
        }
      }
    }
    if (count === 0) {
      row = document.createElement("tr");
      cell = document.createElement("td");
      cell.colSpan = "3";
      cell.innerHTML = "No results available";
      row.appendChild(cell);
      table.appendChild(row);
    }
  } else if (count === 0 && overlay.allowGeneralSearch()) {
    row = document.createElement("tr");
    cell = document.createElement("td");
    cell.colSpan = "3";
    cell.innerHTML = "Search for available targets";
    row.appendChild(cell);
    table.appendChild(row);
  }
  if (this.isOverlayFullView(overlay.getName()) && data["__FULL__"] === undefined) {
    row = document.createElement("tr");
    cell = document.createElement("td");
    cell.colSpan = "3";
    cell.appendChild(this.createWaitingContentDiv());
    row.appendChild(cell);
    table.appendChild(row);
  }

  if (count === 0 && !overlay.allowGeneralSearch() && !this.isOverlayFullView(overlay.getName())) {
    result = null;
  }
  if (result !== null) {
    result.appendChild(table);
  }
  return result;
};

/**
 * Creates and returns DOM div for chemical overlay information.
 * 
 * @param data
 *          data taken from chemical overlay
 * @returns DOM element with a div for comment overlay information
 */
AbstractInfoWindow.prototype._createChemicalInfoDiv = function(overlay, data) {
  return this._createTargetInfoDiv(overlay, data, "Interacting chemicals");
};

/**
 * Creates and returns DOM div for mi rna overlay information.
 * 
 * @param data
 *          data taken from mi rna overlay
 * @returns DOM element with a div for comment overlay information
 */
AbstractInfoWindow.prototype._createMiRnaInfoDiv = function(overlay, data) {
  return this._createTargetInfoDiv(overlay, data, "Interacting Micro RNAs");
};

/**
 * Abstract method returning identifier of the object for which this window was
 * created.
 */
AbstractInfoWindow.prototype.getElementId = function() {
  throw new Error("Not implemented");
};

/**
 * Abstract method returning type of the object for which this window was
 * created.
 */
AbstractInfoWindow.prototype.getElementType = function() {
  throw new Error("Not implemented");
};

module.exports = AbstractInfoWindow;
