"use strict";

var Helper = require('../Helper');

var AliasMarker = require('../../../main/js/map/marker/AliasMarker');
var AbstractCustomMap = require('../../../main/js/map/AbstractCustomMap');
var PointData = require('../../../main/js/map/data/PointData');
var PointMarker = require('../../../main/js/map/marker/PointMarker');

var chai = require('chai');
var assert = chai.assert;

var logger = require("../logger");

describe('AbstractCustomMap', function() {

  var helper;
  before(function() {
    helper = new Helper();
  });

  it("Constructor", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    var mockObject = new AbstractCustomMap(model, options);

    assert.ok(mockObject.getId());
    assert.ok(mockObject.getModel());
    assert.ok(mockObject.getSelectedLayoutOverlays());

    assert.ok(mockObject.pixelOrigin_);
    assert.ok(mockObject.pixelsPerLonDegree_);
    assert.ok(mockObject.pixelsPerLonRadian_);
    assert.ok(mockObject.zoomFactor);

    assert.equal(mockObject.getAliasInfoWindowById(-1), null);
    assert.equal(mockObject.getPointInfoWindowById(-1), null);
    assert.equal(mockObject.getReactionInfoWindowById(-1), null);

    assert.equal(logger.getWarnings().length, 0);
  });

  it("setupLayouts", function() {
    var model = helper.createModel();
    var layout = model.getLayouts()[0];
    var options = helper.createCustomMapOptions();

    var mockObject = new AbstractCustomMap(model, options);

    var gmap = helper.createGoogleMap();

    mockObject.setGoogleMap(gmap);

    mockObject.setupLayouts();
    assert.ok(mockObject.getGoogleMap().getMapTypeId().indexOf(layout.getId()) >= 0);
  });

  it("creatMapOptions", function() {
    var mockObject = helper.createAbstractCustomMap();
    assert.ok(mockObject.creatMapOptions());

  });

  it("createTypeOptions", function() {
    var layoutId = 40;
    var layoutName = 'some layout name';
    var layoutDirectory = 'dire45';

    var model = helper.createModel();
    var layoutDataObj = {
      idObject : layoutId,
      name : layoutName,
      directory : layoutDirectory
    };
    model.addLayout(layoutDataObj);

    var options = helper.createCustomMapOptions();

    var mockObject = new AbstractCustomMap(model, options);

    var result = mockObject.createTypeOptions(layoutDataObj);

    assert.ok(result);

  });

  it("setMaxZoomLevel", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setMaxZoom(4);

    var mockObject = new AbstractCustomMap(model, options);

    var gmap = helper.createGoogleMap();

    mockObject.setGoogleMap(gmap);
    mockObject.setMaxZoomLevel();

    assert.equal(model.getMaxZoom(), mockObject.getGoogleMap().getZoom());
  });

  it("getMouseLatLng", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setMaxZoom(4);

    var mockObject = new AbstractCustomMap(model, options);

    var gmap = new google.maps.Map(testDiv, {});
    gmap.setCenter(new google.maps.LatLng(1, 2));

    mockObject.setGoogleMap(gmap);

    var coords = mockObject.getMouseLatLng();

    assert.ok(coords);
    assert.ok(coords instanceof google.maps.LatLng);
  });

  it("fromPixelsToPoint 1", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setWidth(256);
    model.setHeight(256);
    model.setMinZoom(2);
    model.setMaxZoom(2);

    var mockObject = new AbstractCustomMap(model, options);

    var pointDist = {
      x : 120,
      y : 140
    };

    var normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 2);

    assert.ok(normalizedPoint);
    assert.equal(normalizedPoint.x, pointDist.x);
    assert.equal(normalizedPoint.y, pointDist.y);

  });

  it("fromPixelsToPoint 2", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setWidth(512);
    model.setHeight(512);
    model.setMinZoom(2);
    model.setMaxZoom(3);

    var mockObject = new AbstractCustomMap(model, options);

    var pointDist = {
      x : 120,
      y : 140
    };

    var normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 2);

    assert.ok(normalizedPoint);
    assert.equal(2 * pointDist.x, normalizedPoint.x);
    assert.equal(2 * pointDist.y, normalizedPoint.y);

    normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 3);

    assert.ok(normalizedPoint);
    assert.equal(pointDist.x, normalizedPoint.x);
    assert.equal(pointDist.y, normalizedPoint.y);

  });

  it("fromPixelsToPoint 3", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setWidth(512);
    model.setHeight(512);
    model.setMinZoom(0);
    model.setMaxZoom(3);

    var mockObject = new AbstractCustomMap(model, options);

    var pointDist = {
      x : 120,
      y : 140
    };

    var normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 2);

    assert.ok(normalizedPoint);
    assert.equal(0.5 * pointDist.x, normalizedPoint.x);
    assert.equal(0.5 * pointDist.y, normalizedPoint.y);

    normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 3);

    assert.ok(normalizedPoint);
    assert.equal(0.25 * pointDist.x, normalizedPoint.x);
    assert.equal(0.25 * pointDist.y, normalizedPoint.y);

  });

  it("registerMapClickEvents", function() {
    var mockObject = helper.createAbstractCustomMap();

    var gmap = new google.maps.Map(testDiv, {});

    mockObject.setGoogleMap(gmap);
    // mock function to not put error log on console
    mockObject.getTopMap = function() {
      return null;
    };

    mockObject.registerMapClickEvents();
  });

  it("latLngToTile", function() {
    var mockObject = helper.createAbstractCustomMap();

    var latLng = new google.maps.LatLng(0, 1);

    var tileCoordinates = mockObject.latLngToTile(latLng, 3);
    assert.ok(tileCoordinates);
    assert.ok(tileCoordinates instanceof google.maps.Point);

  });

  it("_turnOnOffDrawing", function() {
    var mockObject = helper.createAbstractCustomMap();

    var gmap = new google.maps.Map(testDiv, {});

    mockObject.setGoogleMap(gmap);

    // mock function to not put error log on console
    mockObject.getTopMap = function() {
      return null;
    };

    assert.equal(mockObject.isDrawingOn(), false);
    mockObject._turnOnOffDrawing();
    assert.ok(mockObject.isDrawingOn());
    mockObject._turnOnOffDrawing();
    assert.equal(mockObject.isDrawingOn(), false);
  });

  it("areaToString", function() {
    var mockObject = helper.createAbstractCustomMap();

    var triangleCoords = [ new google.maps.LatLng(25.774, -80.190), new google.maps.LatLng(18.466, -66.118),
        new google.maps.LatLng(32.321, -64.757), new google.maps.LatLng(25.774, -80.190), ];

    // Construct the polygon.
    var bermudaTriangle = new google.maps.Polygon({
      paths : triangleCoords,
      strokeColor : '#FF0000',
      strokeOpacity : 0.8,
      strokeWeight : 2,
      fillColor : '#FF0000',
      fillOpacity : 0.35
    });

    assert.ok(mockObject.areaToString(bermudaTriangle));

  });

  it("getVisibleLayoutNames", function() {
    var mockObject = helper.createAbstractCustomMap();

    var layout1 = helper.createLayout();
    var layout2 = helper.createLayout();

    mockObject.getModel().addLayout(layout1);
    mockObject.getModel().addLayout(layout2);
    // create mock function to simulate custom map parent
    mockObject.getTopMap = function() {
      var tmpObj = {
        getSelectedLayouts : function() {
          return [ layout1.getId(), layout2.getId() ];
        },
      };
      return tmpObj;
    };

    return mockObject.getVisibleLayoutNames().then(function(data) {
      assert.equal(2, data.length);
      assert.equal(layout1.getName(), data[0]);
      assert.equal(layout2.getName(), data[1]);

    });

  });

  it("Point - Lat,Lng conversion", function() {

    var mockObject = helper.createAbstractCustomMap();

    var x = 19920;
    var y = 11040;
    var point = new google.maps.Point(x, y);

    var coord = mockObject.fromPointToLatLng(point);

    var point2 = mockObject.fromLatLngToPoint(coord);

    assert.closeTo(point2.x, point.x, 0.5, "X coordinate is invalid after transformation");
    assert.closeTo(point2.y, point.y, 0.5, "Y coordinate is invalid after transformation");

  });

  it("_openInfoWindowForMarker(AliasMarker)", function() {
    var mockObject = helper.createCustomMap();

    var data = helper.createAlias();
    var aliasMarker = new AliasMarker(data.getId(), "empty.png", data, mockObject);

    assert.equal(null, mockObject.getAliasInfoWindowById(data.getId()));
    mockObject._openInfoWindowForMarker(aliasMarker);
    assert.ok(mockObject.getAliasInfoWindowById(data.getId()));

  });
  it("_openInfoWindowForMarker(PointMarker)", function() {

    var mockObject = helper.createAbstractCustomMap();

    mockObject.getTopMap = function() {
      var mockObj = {
        getOverlayDataForPoint : function() {
          return [];
        },
      };
      return mockObj;
    };

    var point = new google.maps.Point(2, 3.45);
    var pointData = new PointData(point);

    var pointMarker = new PointMarker(pointData, "empty.png", mockObject);

    assert.equal(null, mockObject.getPointInfoWindowById(pointData.getId()));
    mockObject._openInfoWindowForMarker(pointMarker);
    assert.ok(mockObject.getPointInfoWindowById(pointData.getId()));

  });

  it("_openInfoWindowForReaction", function() {
    var mockObject = helper.createAbstractCustomMap();

    var reaction = helper.createReaction(mockObject);

    // mock the behaviour of the map
    mockObject.getTopMap = function() {
      return mockObject;
    };
    mockObject.getOverlayDataForReaction = function() {
      return [];
    };

    mockObject.getModel().addReaction(reaction);

    assert.equal(null, mockObject.getReactionInfoWindowById(reaction.getId()));

    return mockObject._openInfoWindowForReaction(reaction.getId()).then(function() {
      assert.ok(mockObject.getReactionInfoWindowById(reaction.getId()));
    });

  });
});
